﻿Option Strict On

Public Class Kinect_RingBuffer
    Public Structure RingBufferData

        Public Property ColorPixelData() As Byte()
        Public Property DepthPixelData() As Short()
        Public Property PlayerIndexData() As Integer()

        Public Sub InitArray(ColorStream_FramePixelDataLength As Integer, DepthStream_FramePixelDataLength As Integer)
            ColorPixelData = New Byte(ColorStream_FramePixelDataLength - 1) {}
            DepthPixelData = New Short(DepthStream_FramePixelDataLength - 1) {}
            PlayerIndexData = New Integer(ColorStream_FramePixelDataLength - 1) {}
        End Sub


        ''' RGB配列をコピー
        ''' 'Array.Copy(コピーするデータを格納しているArray,データのコピー先となるArry,コピーする要素の数を表す32ビット整数)
        Public Sub copy_framedata(ByRef value As Byte())
            System.Array.Copy(value, ColorPixelData, value.Length)
        End Sub

        ''' Depth配列をコピー
        Public Sub copymyDepthPixelData(value As Short())
            System.Array.Copy(value, DepthPixelData, value.Length)
        End Sub

        ''' PlayerIndex配列をコピー
        Public Sub copy_PlayerIndexData(value As Integer())
            System.Array.Copy(value, PlayerIndexData, value.Length)
        End Sub

    End Structure


    Public Structure RingBuffer
        Dim myRingBufferData As RingBufferData()

        Property max_buffer_frame() As Integer
        Property buffer_index() As Integer

        Private Function get_index(frameNo As Integer) As Integer
            Dim no As Integer
            If frameNo < max_buffer_frame Then
                no = frameNo
            Else
                no = frameNo - max_buffer_frame
            End If
            If frameNo < 0 Then
                no = max_buffer_frame + frameNo
            End If
            Return no
        End Function

        Public Function get_rgb_frame(frameNo As Integer) As Byte()
            If myRingBufferData Is Nothing Then
                Return Nothing
            End If
            Return myRingBufferData(get_index(frameNo)).ColorPixelData
        End Function

        Public Function get_depth_frame(frameNo As Integer) As Short()
            If myRingBufferData Is Nothing Then
                Return Nothing
            End If
            Return myRingBufferData(get_index(frameNo)).DepthPixelData
        End Function

        Public Function get_PlayerIndexData(frameNo As Integer) As Integer()
            If myRingBufferData Is Nothing Then
                Return Nothing
            End If
            Return myRingBufferData(get_index(frameNo)).PlayerIndexData
        End Function

        Public Function init_ringbuffer(buffer_sec As Integer, ColorStream_FramePixelDataLength As Integer, DepthStream_FramePixelDataLength As Integer) As Boolean
            Try
                buffer_index = 0
                max_buffer_frame = 30 * buffer_sec
                '30fps*second
                myRingBufferData = New RingBufferData(max_buffer_frame - 1) {}

                For i As Integer = 0 To max_buffer_frame - 1
                    myRingBufferData(i) = New RingBufferData()
                    myRingBufferData(i).InitArray(ColorStream_FramePixelDataLength, DepthStream_FramePixelDataLength)
                Next

                Return True
            Catch ex As Exception
                Return False
            End Try
        End Function

        Public Sub save_framedata(value As Byte())
            myRingBufferData(buffer_index).copy_framedata(value)
        End Sub


        Public Sub save_depthdata(value As Short())
            myRingBufferData(buffer_index).copymyDepthPixelData(value)
        End Sub

        Public Sub save_playerIndexdata(value As Integer())
            myRingBufferData(buffer_index).copy_PlayerIndexData(value)
        End Sub


        Public Sub set_nextframe()
            If buffer_index >= max_buffer_frame - 1 Then
                buffer_index = 0
            Else
                buffer_index = buffer_index + 1
            End If
        End Sub
    End Structure
End Class

