﻿Option Strict On
Imports Microsoft.Kinect
Imports RingBufferLib.Kinect_RingBuffer
Class MainWindow

    Dim BytesPerPixel As Integer = CInt(PixelFormats.Bgr32.BitsPerPixel / 8) '4
    Const myPlayer As Integer = 7

    Dim Kinect As KinectSensor

    Dim myDepthPixelData As Short()
    Dim myColorPixelData As Byte()


    Const bufferSecond As Integer = 3

    Dim screenImageStride As Integer


    Dim myImageSize As Int32Rect
    Dim myRingBuffer As RingBuffer

    Dim overrayBitmap As WriteableBitmap
    Dim myScreenImageRect As Int32Rect

    Private Sub MainWindow_Loaded(sender As Object, e As System.Windows.RoutedEventArgs) Handles Me.Loaded
        init_kinect()
    End Sub

    Private Sub init_kinect()
        Try
            If KinectSensor.KinectSensors.Count = 0 Then
                MessageBox.Show("KINECTが接続されておりません。")
                Exit Sub
            Else
                Kinect = KinectSensor.KinectSensors(0)
            End If

            Kinect.DepthStream.Enable(DepthImageFormat.Resolution320x240Fps30) '640×480では重くて動かない。
            Kinect.ColorStream.Enable(ColorImageFormat.RgbResolution640x480Fps30)
            Kinect.SkeletonStream.Enable()

            Dim myDepthStream As DepthImageStream = Kinect.DepthStream
            myDepthPixelData = New Short(Kinect.DepthStream.FramePixelDataLength - 1) {}
            myColorPixelData = New Byte(Kinect.ColorStream.FramePixelDataLength - 1) {}


            overrayBitmap = New WriteableBitmap(myDepthStream.FrameWidth, myDepthStream.FrameHeight, 96, 96, PixelFormats.Bgra32, Nothing)
            myScreenImageRect = New Int32Rect(0, 0, CInt(Math.Ceiling(overrayBitmap.Width)), CInt(Math.Ceiling(overrayBitmap.Height)))

            myRingBuffer.init_ringbuffer(bufferSecond, Kinect.ColorStream.FramePixelDataLength, Kinect.DepthStream.FramePixelDataLength)
            AddHandler Kinect.AllFramesReady, AddressOf kinect_AllFramesReady

            Kinect.Start()

            myImageSize.Width = Kinect.ColorStream.FrameWidth
            myImageSize.Height = Kinect.ColorStream.FrameHeight

        Catch ex As Exception
            MessageBox.Show(ex.Message)
            Exit Sub
        End Try
    End Sub


    Public Sub StopKinect()
        If Kinect Is Nothing = False Then
            If Kinect.IsRunning = True Then
                RemoveHandler Kinect.AllFramesReady, AddressOf kinect_AllFramesReady
                Kinect.Stop()
                Kinect.Dispose()
            End If
        End If
    End Sub

    Private Sub kinect_AllFramesReady(sender As Object, e As AllFramesReadyEventArgs)
        Using colorFrame As ColorImageFrame = e.OpenColorImageFrame()
            Using depthFrame As DepthImageFrame = e.OpenDepthImageFrame()
                Using skeletonFrame As SkeletonFrame = e.OpenSkeletonFrame()
                    SaveBuffer(colorFrame, depthFrame, skeletonFrame)
                End Using
            End Using
        End Using
    End Sub


    Private Sub SaveBuffer(colorFrame As ColorImageFrame, depthFrame As DepthImageFrame, skeletonFrame As SkeletonFrame)
        If Kinect Is Nothing OrElse depthFrame Is Nothing OrElse colorFrame Is Nothing OrElse skeletonFrame Is Nothing Then
            Return
        End If

        Dim myColorStream As ColorImageStream = Kinect.ColorStream
        Dim myDepthStream As DepthImageStream = Kinect.DepthStream

        screenImageStride = colorFrame.BytesPerPixel * depthFrame.Width
        Dim colorStride As Integer = colorFrame.BytesPerPixel * colorFrame.Width

        Dim ImageIndex As Integer = 0

        depthFrame.CopyPixelDataTo(myDepthPixelData)
        colorFrame.CopyPixelDataTo(myColorPixelData)


        Dim colorPoint As ColorImagePoint() = New ColorImagePoint(depthFrame.PixelDataLength - 1) {}
        Dim depthPixel As Short() = New Short(depthFrame.PixelDataLength - 1) {}
        Kinect.MapDepthFrameToColorFrame(depthFrame.Format, depthPixel, colorFrame.Format, colorPoint)


        Dim byteRoom As Byte() = New Byte(depthFrame.Height * screenImageStride - 1) {} 'New Byte(409600 - 1) {}に同じ
        Dim bytePlayer As Byte() = New Byte(depthFrame.Height * screenImageStride - 1) {} 'New Byte(409600 - 1) {}に同じ


        Dim depth As Double() = New Double(depthFrame.Height * screenImageStride - 1) {} ' New Double(307200 - 1) {}に同じ
        Dim playerIndexArray As Integer() = New Integer(depthFrame.Height * screenImageStride - 1) {} ' New Double(307200 - 1) {}に同じ


        For depthY As Integer = 0 To depthFrame.Height - 1
            Dim depthX As Integer = 0
            While depthX < depthFrame.Width
                Dim depthPixelIndex As Integer = depthX + (depthY * depthFrame.Width)
                Dim playerIndex As Integer = myDepthPixelData(depthPixelIndex) And DepthImageFrame.PlayerIndexBitmask
                Dim x As Integer = colorPoint(depthPixelIndex).X
                Dim y As Integer = colorPoint(depthPixelIndex).Y

                Dim colorPixelIndex As Integer = (x * colorFrame.BytesPerPixel) + (y * colorStride)

                If playerIndex <> 0 Then
                    depth(ImageIndex) = myDepthPixelData(depthPixelIndex) >> DepthImageFrame.PlayerIndexBitmaskWidth
                    playerIndexArray(ImageIndex) = playerIndex

                    bytePlayer(ImageIndex) = myColorPixelData(colorPixelIndex)
                    'Blue    
                    bytePlayer(ImageIndex + 1) = myColorPixelData(colorPixelIndex + 1)
                    'Green
                    bytePlayer(ImageIndex + 2) = myColorPixelData(colorPixelIndex + 2)
                    'Red
                    bytePlayer(ImageIndex + 3) = &HFF
                    'Alpha
                End If
                depthX = depthX + 1
                ImageIndex = ImageIndex + colorFrame.BytesPerPixel
            End While
        Next

        myRingBuffer.save_framedata(bytePlayer)
        myRingBuffer.save_playerIndexdata(playerIndexArray)
        myRingBuffer.set_nextframe()
        RenderScreen2()

    End Sub

    Private Sub RenderScreen2()
        If screenImageStride = 0 Then
            Return
        End If
        Dim bytePlayer As Byte() = New Byte(CInt(myImageSize.Height) * screenImageStride - 1) {} 'New Byte(819200 - 1) {}に同じ

        Dim frame As Integer = 15

        For i As Integer = 0 To myPlayer - 1
            Dim iframe As Integer

            If i = 0 Then
                iframe = myRingBuffer.buffer_index
            Else
                iframe = myRingBuffer.buffer_index + (frame * i - 1)
            End If
            Dim colorPixelData As Byte() = myRingBuffer.get_rgb_frame(iframe)
            Dim PlayerIndex As Integer() = myRingBuffer.get_PlayerIndexData(iframe)
            If colorPixelData Is Nothing Then
                Continue For
            End If

            Dim index As Integer = 0
            While index < colorPixelData.Length
                If PlayerIndex(index) > 0 Then
                    bytePlayer(index) = colorPixelData(index)
                    bytePlayer(index + 1) = colorPixelData(index + 1)
                    bytePlayer(index + 2) = colorPixelData(index + 2)
                    bytePlayer(index + 3) = colorPixelData(index + 3)
                End If
                index = index + BytesPerPixel
            End While
        Next

        overrayBitmap.WritePixels(myScreenImageRect, bytePlayer, screenImageStride, 0)
        human_image1.Source = overrayBitmap

    End Sub

    Private Sub MainWindow_Closing(sender As Object, e As System.ComponentModel.CancelEventArgs) Handles Me.Closing
        StopKinect()
    End Sub
End Class

